@@ -1,6 +1,6 @@
 /*
      This file is part of libextractor.
-     Copyright (C) 2008, 2012 Heikki Lindholm and Christian Grothoff
+     Copyright Copyright (C) 2008, 2012 Heikki Lindholm and Christian Grothoff
 
      libextractor is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with libextractor; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
  */
 /**
  * @file thumbnailffmpeg_extractor.c
@@ -25,14 +25,14 @@
  * thumbnail of images and videos using the ffmpeg libs.
  *
  * This is a thumbnail extractor using the ffmpeg libraries that will eventually
- * support extracting thumbnails from both image and video files. 
+ * support extracting thumbnails from both image and video files.
  *
  * Note that ffmpeg has a few issues:
  * (1) there are no recent official releases of the ffmpeg libs
  * (2) ffmpeg has a history of having security issues (parser is not robust)
  *
  *  So this plugin cannot be recommended for system with high security
- *requirements. 
+ *requirements.
  */
 #include "platform.h"
 #include "extractor.h"
@@ -59,15 +59,40 @@
 #include <ffmpeg/swscale.h>
 #endif
 
+#if USE_JPEG
+#ifdef PIX_FMT_YUVJ420P
+#define PIX_OUTPUT_FORMAT PIX_FMT_YUVJ420P
+#else
+#define PIX_OUTPUT_FORMAT AV_PIX_FMT_YUVJ420P
+#endif
+#else
+#ifdef PIX_FMT_RGB24
+#define PIX_OUTPUT_FORMAT PIX_FMT_RGB24
+#else
+#define PIX_OUTPUT_FORMAT AV_PIX_FMT_RGB24
+#endif
+#endif
+
 /**
- * Set to 1 to enable debug output.
- */ 
+ * Set to 1 to use JPEG, PNG otherwise
+ */
+#define USE_JPEG 1
+
+/**
+ * Set to 1 to enable a output file for testing.
+ */
+#define OUTPUT_FILE 0
+
+
+/**
+ * Set to 1 to use jpeg.
+ */
 #define DEBUG 0
 
 /**
  * max dimension in pixels for the thumbnail.
  */
-#define MAX_THUMB_DIMENSION 128         
+#define MAX_THUMB_DIMENSION 128
 
 /**
  * Maximum size in bytes for the thumbnail.
@@ -141,7 +166,7 @@ seek_cb (void *opaque,
  *
  * @param src_width source image width
  * @param src_height source image height
- * @param src_stride 
+ * @param src_stride
  * @param src_pixfmt
  * @param src_data source data
  * @param dst_width desired thumbnail width
@@ -150,13 +175,13 @@ seek_cb (void *opaque,
  * @param output_max_size maximum size of result that is allowed
  * @return the number of bytes used, 0 on error
  */
-static size_t 
-create_thumbnail (int src_width, int src_height, 
+static size_t
+create_thumbnail (AVCodecContext *pCodecCtx, int src_width, int src_height,
 		  int src_stride[],
-		  enum PixelFormat src_pixfmt, 
+		  enum AVPixelFormat src_pixfmt,
 		  const uint8_t * const src_data[],
 		  int dst_width, int dst_height,
-		  uint8_t **output_data, 
+		  uint8_t **output_data,
 		  size_t output_max_size)
 {
   AVCodecContext *encoder_codec_ctx;
@@ -168,29 +193,37 @@ create_thumbnail (int src_width, int src_height,
   uint8_t *encoder_output_buffer;
   size_t encoder_output_buffer_size;
   int err;
-  
+
   AVPacket pkt;
   av_init_packet(&pkt);
   pkt.data = NULL;
   pkt.size = 0;
   int gotPacket;
-
-  if (NULL == (encoder_codec = avcodec_find_encoder_by_name ("png")))
+#if USE_JPEG
+	#if LIBAVCODEC_BUILD >= AV_VERSION_INT(54,25,0)
+	if (NULL == (encoder_codec = avcodec_find_encoder ( AV_CODEC_ID_MJPEG )))
+	#else
+	if (NULL == (encoder_codec = avcodec_find_encoder ( CODEC_ID_MJPEG )))
+	#endif
+ #else
+ if (NULL == (encoder_codec = avcodec_find_encoder_by_name ("png")))
+ #endif
     {
 #if DEBUG
       fprintf (stderr,
-	       "Couldn't find a PNG encoder\n");
+	       "Couldn't find a encoder\n");
 #endif
       return 0;
     }
 
   /* NOTE: the scaler will be used even if the src and dst image dimensions
    * match, because the scaler will also perform colour space conversion */
-  if (NULL == 
+  if (NULL ==
       (scaler_ctx =
        sws_getContext (src_width, src_height, src_pixfmt,
-		       dst_width, dst_height, PIX_FMT_RGB24, 
-		       SWS_BILINEAR, NULL, NULL, NULL)))
+		       dst_width, dst_height,
+                       PIX_OUTPUT_FORMAT,
+                       SWS_BILINEAR, NULL, NULL, NULL)))
     {
 #if DEBUG
       fprintf (stderr,
@@ -199,7 +232,12 @@ create_thumbnail (int src_width, int src_height,
       return 0;
     }
 
-  if (NULL == (dst_frame = avcodec_alloc_frame ()))
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+  dst_frame = av_frame_alloc ();
+#else
+  dst_frame = avcodec_alloc_frame();
+#endif
+  if (NULL == dst_frame)
     {
 #if DEBUG
       fprintf (stderr,
@@ -209,23 +247,29 @@ create_thumbnail (int src_width, int src_height,
       return 0;
     }
   if (NULL == (dst_buffer =
-	       av_malloc (avpicture_get_size (PIX_FMT_RGB24, dst_width, dst_height))))
+	       av_malloc (avpicture_get_size (PIX_OUTPUT_FORMAT,
+                                              dst_width, dst_height))))
     {
 #if DEBUG
       fprintf (stderr,
                "Failed to allocate the destination image buffer\n");
 #endif
-      av_free (dst_frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+      av_frame_free (&dst_frame);
+#else
+      avcodec_free_frame (&dst_frame);
+#endif
       sws_freeContext (scaler_ctx);
       return 0;
     }
   avpicture_fill ((AVPicture *) dst_frame, dst_buffer,
-                  PIX_FMT_RGB24, dst_width, dst_height);
+                  PIX_OUTPUT_FORMAT,
+                  dst_width, dst_height);
   sws_scale (scaler_ctx,
-             src_data, 
+             src_data,
              src_stride,
-             0, src_height, 
-             dst_frame->data, 
+             0, src_height,
+             dst_frame->data,
              dst_frame->linesize);
 
   encoder_output_buffer_size = output_max_size;
@@ -236,7 +280,11 @@ create_thumbnail (int src_width, int src_height,
                "Failed to allocate the encoder output buffer\n");
 #endif
       av_free (dst_buffer);
-      av_free (dst_frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+      av_frame_free (&dst_frame);
+#else
+      avcodec_free_frame (&dst_frame);
+#endif
       sws_freeContext (scaler_ctx);
       return 0;
     }
@@ -249,13 +297,35 @@ create_thumbnail (int src_width, int src_height,
 #endif
       av_free (encoder_output_buffer);
       av_free (dst_buffer);
-      av_free (dst_frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+      av_frame_free (&dst_frame);
+#else
+      avcodec_free_frame (&dst_frame);
+#endif
       sws_freeContext (scaler_ctx);
       return 0;
     }
   encoder_codec_ctx->width = dst_width;
   encoder_codec_ctx->height = dst_height;
-  encoder_codec_ctx->pix_fmt = PIX_FMT_RGB24;
+#if USE_JPEG
+  encoder_codec_ctx->bit_rate      = pCodecCtx->bit_rate;
+#if LIBAVCODEC_BUILD >= AV_VERSION_INT(54,25,0)
+  encoder_codec_ctx->codec_id      = AV_CODEC_ID_MJPEG;
+#else
+  encoder_codec_ctx->codec_id      = CODEC_ID_MJPEG;
+#endif
+#if LIBAVCODEC_BUILD >= AV_VERSION_INT(53,35,0)
+  encoder_codec_ctx->codec_type    = AVMEDIA_TYPE_VIDEO;
+#else
+  encoder_codec_ctx->codec_type    = CODEC_TYPE_VIDEO;
+#endif
+  encoder_codec_ctx->time_base.num = pCodecCtx->time_base.num;
+  encoder_codec_ctx->time_base.den = pCodecCtx->time_base.den;
+  encoder_codec_ctx->pix_fmt = PIX_OUTPUT_FORMAT;
+#else
+  encoder_codec_ctx->pix_fmt = PIX_OUTPUT_FORMAT;
+#endif
+
   opts = NULL;
   if (avcodec_open2 (encoder_codec_ctx, encoder_codec, &opts) < 0)
     {
@@ -263,17 +333,36 @@ create_thumbnail (int src_width, int src_height,
       fprintf (stderr,
                "Failed to open the encoder\n");
 #endif
-      av_free (encoder_codec_ctx);
+      avcodec_free_context (&encoder_codec_ctx);
       av_free (encoder_output_buffer);
       av_free (dst_buffer);
-      av_free (dst_frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+      av_frame_free (&dst_frame);
+#else
+      avcodec_free_frame (&dst_frame);
+#endif
       sws_freeContext  (scaler_ctx);
       return 0;
     }
-	
-			   
+
+
+
+#ifdef USE_JPEG
+#if FF_API_MPV_OPT
+   encoder_codec_ctx->mb_lmin        = encoder_codec_ctx->lmin = encoder_codec_ctx->qmin * FF_QP2LAMBDA;
+   encoder_codec_ctx->mb_lmax        = encoder_codec_ctx->lmax = encoder_codec_ctx->qmax * FF_QP2LAMBDA;
+#else
+   encoder_codec_ctx->mb_lmin = encoder_codec_ctx->qmin * FF_QP2LAMBDA;
+   encoder_codec_ctx->mb_lmax = encoder_codec_ctx->qmax * FF_QP2LAMBDA;
+#endif
+   encoder_codec_ctx->flags          = CODEC_FLAG_QSCALE;
+   encoder_codec_ctx->global_quality = encoder_codec_ctx->qmin * FF_QP2LAMBDA;
+
+   dst_frame->pts     = 1;
+   dst_frame->quality = encoder_codec_ctx->global_quality;
+#endif
+
 #if LIBAVCODEC_BUILD >= AV_VERSION_INT(54,25,0)
-  //err = encode_frame (encoder_codec_ctx, dst_frame);							  
   err = avcodec_encode_video2 (encoder_codec_ctx,
                               &pkt,
                                dst_frame, &gotPacket);
@@ -284,20 +373,24 @@ create_thumbnail (int src_width, int src_height,
   memcpy(encoder_output_buffer,pkt.data, pkt.size);
 
   av_free_packet(&pkt);
-							  
-							  
+
+
 #else
   err = avcodec_encode_video (encoder_codec_ctx,
                               encoder_output_buffer,
                               encoder_output_buffer_size, dst_frame);
-  
+
 #endif
 cleanup:
   av_dict_free (&opts);
   avcodec_close (encoder_codec_ctx);
-  av_free (encoder_codec_ctx);
+  avcodec_free_context (&encoder_codec_ctx);
   av_free (dst_buffer);
-  av_free (dst_frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+  av_frame_free (&dst_frame);
+#else
+  avcodec_free_frame (&dst_frame);
+#endif
   sws_freeContext (scaler_ctx);
   *output_data = encoder_output_buffer;
 
@@ -307,7 +400,7 @@ cleanup:
 
 
 /**
- * calculate the thumbnail dimensions, taking pixel aspect into account 
+ * calculate the thumbnail dimensions, taking pixel aspect into account
  *
  * @param src_width source image width
  * @param src_height source image height
@@ -316,7 +409,7 @@ cleanup:
  * @param dst_width desired thumbnail width (set)
  * @param dst_height desired thumbnail height (set)
   */
-static void 
+static void
 calculate_thumbnail_dimensions (int src_width,
 				int src_height,
 				int src_sar_num,
@@ -348,7 +441,7 @@ calculate_thumbnail_dimensions (int src_width,
     *dst_height = 1;
 #if DEBUG
   fprintf (stderr,
-           "Thumbnail dimensions: %d %d\n", 
+           "Thumbnail dimensions: %d %d\n",
            *dst_width, *dst_height);
 #endif
 }
@@ -406,18 +499,23 @@ extract_image (ENUM_CODEC_ID image_codec_id,
       fprintf (stderr,
 	       "Failed to open image codec\n");
 #endif
-      av_free (codec_ctx);
+      avcodec_free_context (&codec_ctx);
       return;
     }
   av_dict_free (&opts);
-  if (NULL == (frame = avcodec_alloc_frame ()))
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+  frame = av_frame_alloc ();
+#else
+  frame = avcodec_alloc_frame();
+#endif
+  if (NULL == frame)
     {
 #if DEBUG
       fprintf (stderr,
                "Failed to allocate frame\n");
 #endif
       avcodec_close (codec_ctx);
-      av_free (codec_ctx);
+      avcodec_free_context (&codec_ctx);
       return;
     }
 
@@ -441,9 +539,13 @@ extract_image (ENUM_CODEC_ID image_codec_id,
       fprintf (stderr,
 	       "Failed to decode a complete frame\n");
 #endif
-      av_free (frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+      av_frame_free (&frame);
+#else
+      avcodec_free_frame (&frame);
+#endif
       avcodec_close (codec_ctx);
-      av_free (codec_ctx);
+      avcodec_free_context (&codec_ctx);
       return;
     }
   calculate_thumbnail_dimensions (codec_ctx->width, codec_ctx->height,
@@ -451,8 +553,8 @@ extract_image (ENUM_CODEC_ID image_codec_id,
                                   codec_ctx->sample_aspect_ratio.den,
                                   &thumb_width, &thumb_height);
 
-  err = create_thumbnail (codec_ctx->width, codec_ctx->height,
-                          frame->linesize, codec_ctx->pix_fmt, 
+  err = create_thumbnail (codec_ctx, codec_ctx->width, codec_ctx->height,
+                          frame->linesize, codec_ctx->pix_fmt,
 			  (const uint8_t * const*) frame->data,
                           thumb_width, thumb_height,
                           &encoded_thumbnail, MAX_THUMB_BYTES);
@@ -465,11 +567,34 @@ extract_image (ENUM_CODEC_ID image_codec_id,
 		"image/png",
 		(const char*) encoded_thumbnail,
 		err);
+
+		#if OUTPUT_FILE
+        	FILE *f;
+			#ifdef USE_JPEG
+        	f = fopen("thumb.jpg", "wb");
+			#else
+			f = fopen("thumb.png", "wb");
+			#endif
+            if (!f) {
+                fprintf(stderr, "Could not open %s\n", "file");
+                exit(1);
+            }
+
+        	fwrite(encoded_thumbnail, 1, err, f);
+        	fclose(f);
+
+        #endif
+
+
       av_free (encoded_thumbnail);
     }
-  av_free (frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+  av_frame_free (&frame);
+#else
+  avcodec_free_frame (&frame);
+#endif
   avcodec_close (codec_ctx);
-  av_free (codec_ctx);
+  avcodec_free_context (&codec_ctx);
 }
 
 
@@ -500,7 +625,7 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
   if (NULL == (iob = av_malloc (16 * 1024)))
     return;
   if (NULL == (io_ctx = avio_alloc_context (iob, 16 * 1024,
-					    0, ec, 
+					    0, ec,
 					    &read_cb,
 					    NULL /* no writing */,
 					    &seek_cb)))
@@ -517,10 +642,10 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
   options = NULL;
   if (0 != avformat_open_input (&format_ctx, "<no file>", NULL, &options))
     return;
-  av_dict_free (&options);  
+  av_dict_free (&options);
   if (0 > avformat_find_stream_info (format_ctx, NULL))
     {
- #if DEBUG
+#if DEBUG
       fprintf (stderr,
                "Failed to read stream info\n");
 #endif
@@ -544,12 +669,12 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
           codec = NULL;
           continue;
         }
-      av_dict_free (&options); 
+      av_dict_free (&options);
       video_stream_index = i;
       break;
     }
   if ( (-1 == video_stream_index) ||
-       (0 == codec_ctx->width) || 
+       (0 == codec_ctx->width) ||
        (0 == codec_ctx->height) )
     {
 #if DEBUG
@@ -563,7 +688,12 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
       return;
     }
 
-  if (NULL == (frame = avcodec_alloc_frame ()))
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+  frame = av_frame_alloc ();
+#else
+  frame = avcodec_alloc_frame();
+#endif
+  if (NULL == frame)
     {
 #if DEBUG
       fprintf (stderr,
@@ -574,20 +704,35 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
       av_free (io_ctx);
       return;
     }
-#if DEBUG
+  int duration;
   if (format_ctx->duration == AV_NOPTS_VALUE)
+	{
+	duration = -1;
+#if DEBUG
     fprintf (stderr,
 	     "Duration unknown\n");
-  else
-    fprintf (stderr,
-	     "Duration: %lld\n", 
-	     format_ctx->duration);      
 #endif
-  /* TODO: if duration is known, seek to some better place,
-   * but use 10 sec into stream for now */
-  err = av_seek_frame (format_ctx, -1, 10 * AV_TIME_BASE, 0);
-  if (err >= 0)        
-    avcodec_flush_buffers (codec_ctx);        
+	}
+  else
+  {
+#if DEBUG
+	duration = format_ctx->duration;
+    fprintf (stderr,
+	     "Duration: %lld\n",
+	     format_ctx->duration);
+#endif
+	}
+
+  /* if duration is known, seek to first tried,
+   * else use 10 sec into stream */
+
+  if(-1 != duration)
+	err = av_seek_frame (format_ctx, -1, (duration/3), 0);
+  else
+	err = av_seek_frame (format_ctx, -1, 10 * AV_TIME_BASE, 0);
+
+  if (err >= 0)
+    avcodec_flush_buffers (codec_ctx);
   frame_finished = 0;
 
   while (1)
@@ -616,7 +761,11 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
       fprintf (stderr,
 	       "Failed to decode a complete frame\n");
 #endif
-      av_free (frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+      av_frame_free (&frame);
+#else
+      avcodec_free_frame (&frame);
+#endif
       avcodec_close (codec_ctx);
       avformat_close_input (&format_ctx);
       av_free (io_ctx);
@@ -626,14 +775,15 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
                                   codec_ctx->sample_aspect_ratio.num,
                                   codec_ctx->sample_aspect_ratio.den,
                                   &thumb_width, &thumb_height);
-  err = create_thumbnail (codec_ctx->width, codec_ctx->height,
+
+  err = create_thumbnail (codec_ctx, codec_ctx->width, codec_ctx->height,
                           frame->linesize, codec_ctx->pix_fmt,
 			  (const uint8_t* const *) frame->data,
                           thumb_width, thumb_height,
                           &encoded_thumbnail, MAX_THUMB_BYTES);
   if (err > 0)
     {
-	
+
       ec->proc (ec->cls,
 		"thumbnailffmpeg",
 		EXTRACTOR_METATYPE_THUMBNAIL,
@@ -641,9 +791,29 @@ extract_video (struct EXTRACTOR_ExtractContext *ec)
 		"image/png",
 		(const char*) encoded_thumbnail,
 		err);
+		#if OUTPUT_FILE
+        	FILE *f;
+			#ifdef USE_JPEG
+        	f = fopen("thumb.jpg", "wb");
+			#else
+			f = fopen("thumb.png", "wb");
+			#endif
+            if (!f) {
+                fprintf(stderr, "Could not open %s\n", "file");
+                exit(1);
+            }
+
+        	fwrite(encoded_thumbnail, 1, err, f);
+        	fclose(f);
+
+        #endif
       av_free (encoded_thumbnail);
     }
-  av_free (frame);
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55,28,1)
+  av_frame_free (&frame);
+#else
+  avcodec_free_frame (&frame);
+#endif
   avcodec_close (codec_ctx);
   avformat_close_input (&format_ctx);
   av_free (io_ctx);
@@ -668,9 +838,9 @@ struct MIMEToDecoderMapping
 
 
 /**
- * map MIME image types to an ffmpeg decoder 
+ * map MIME image types to an ffmpeg decoder
  */
-static const struct MIMEToDecoderMapping m2d_map[] = 
+static const struct MIMEToDecoderMapping m2d_map[] =
   {
 
 #if LIBAVCODEC_BUILD >= AV_VERSION_INT(54,25,0)
@@ -699,7 +869,7 @@ static const struct MIMEToDecoderMapping m2d_map[] =
  *
  * @param ec extraction context
  */
-void 
+void
 EXTRACTOR_thumbnailffmpeg_extract_method (struct EXTRACTOR_ExtractContext *ec)
 {
   unsigned int i;
@@ -731,7 +901,7 @@ EXTRACTOR_thumbnailffmpeg_extract_method (struct EXTRACTOR_ExtractContext *ec)
  *
  * @param ec extraction context
  */
-void 
+void
 EXTRACTOR_thumbnail_extract_method (struct EXTRACTOR_ExtractContext *ec)
 {
   EXTRACTOR_thumbnailffmpeg_extract_method (ec);
@@ -746,8 +916,8 @@ EXTRACTOR_thumbnail_extract_method (struct EXTRACTOR_ExtractContext *ec)
  * @param format format string
  * @param ap arguments for format
  */
-static void 
-thumbnailffmpeg_av_log_callback (void* ptr, 
+static void
+thumbnailffmpeg_av_log_callback (void* ptr,
 				 int level,
 				 const char *format,
 				 va_list ap)
@@ -761,7 +931,7 @@ thumbnailffmpeg_av_log_callback (void* ptr,
 /**
  * Initialize av-libs and load magic file.
  */
-void __attribute__ ((constructor)) 
+void __attribute__ ((constructor))
 thumbnailffmpeg_lib_init (void)
 {
   av_log_set_callback (&thumbnailffmpeg_av_log_callback);
@@ -777,8 +947,8 @@ thumbnailffmpeg_lib_init (void)
 /**
  * Destructor for the library, cleans up.
  */
-void __attribute__ ((destructor)) 
-thumbnailffmpeg_ltdl_fini () 
+void __attribute__ ((destructor))
+thumbnailffmpeg_ltdl_fini ()
 {
   if (NULL != magic)
     {
